// "Cave Adventure". Original by Russel Wallace for Commodore 64 and Amiga.
// Ported to TI-89 & TI-92+ by Zeljko Juric, based on Russel's generic C port.
// You need TIGCC crosscompiler with library 2.1 or greater to compile it.
// Use "tigcc -O2 caveadv.c" to compile.

#define SAVE_SCREEN

#include <nostub.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <alloc.h>
#include <compat.h>
#include <graph.h>
#include <kbd.h>
#include <statline.h>
#include <vat.h>

int _ti89,_ti92plus;                // Produce code for both TI-89  & TI-92+

typedef unsigned char ubyte;

#define NUM_VERB 42
#define NUM_NOUN 47
#define NUM_LOC 146
#define NUM_OBJ 33
#define BUFLEN 30
#define NUM_NOUN_SYN 8
#define NUM_VERB_SYN 18
#define FLAGS_SIZE 15    

#define TRUE 1
#define FALSE 0
#define NOWHERE 0
#define NOTHING 0
#define CARRIED 255
#define WORN 254
#define MAXWEIGHT 60

enum {M_BRIEF,M_NORMAL,M_VERBOSE};

enum {V_NORTH=1,V_SOUTH,V_EAST,V_WEST,V_LOOK,V_SCORE,V_HELP,V_OPEN,V_WAVE,
  V_CUT,V_LIGHT,V_OIL,V_EXAMINE,V_SAVE,V_LOAD,V_OFF,V_GET,V_DROP,V_INV,
  V_QUIT,V_CLOSE,V_CLIMB,V_JUMP,V_CROSS,V_SPRAY,V_WEAR,V_REMOVE,V_WATER,
  V_FILL,V_EMPTY,V_THROW,V_UNLOAD,V_FIRE,V_SHOOT,V_KILL,V_BREAK,V_KICK,
  V_FUCK,V_VERBOSE,V_BRIEF,V_NORMAL,V_SWIM};

enum {N_TORCH_UNLIT=1,N_TORCH_LIT,N_MATCHES,N_AXE,N_PLANK,N_DIAMOND,N_OIL,
  N_CAN,N_HACKSAW,N_WEEDKILLER,N_PACKET,N_MASK,N_WATER,N_BUCKET,N_BULLET,
  N_BRACELET,N_GUN_UNLOADED,N_GUN_LOADED,N_RING,N_CRUCIFIX,N_MIRROR,N_PAPER,
  N_KEY,N_STONE,N_STAFF,N_STATUETTE,N_SAFE_LOCKED,N_SAFE_OPEN,N_EMERALD,
  N_FOLDER,N_TOAD,N_TREE,N_FALLEN_TREE,N_GLASS,N_WEEDS,N_WIRE,N_GATE,
  N_WEREWOLF,N_SIGN,N_PLANT,N_OGRE,N_DOOR,N_CHASM,N_STREAM,N_VAMPIRE,
  N_BASILISK,N_ALL};

enum {L_STREAM_EAST=4,L_STREAM_WEST,L_DROPZONE=16,L_TREE=18,L_TREETOP,
  L_BEYOND_GLASS=21,L_GLASS,L_CHASM_WEST=26,L_CHASM_EAST,L_GATE=34,L_WIRE=38,
  L_SAFE=40,L_BASILISK=46,L_DOOR=60,L_WEEDS=64,L_DRAUGHTS=66,L_PLANT=69,
  L_SIGN=71,L_VAMPIRE=73,L_WEREWOLF=81,L_OGRE=87,L_TOP_PLANT=146};

char buffer[BUFLEN]={};                   // Buffer for input of commands
char filename[BUFLEN]="\0ca_save";        // Default save/load filename

char signature[]={0,'C','A',0,'S','A','V',0,0xF8};  // Savefile signature

ubyte flag_base[FLAGS_SIZE]={};           // Game flags

#define mode flag_base[0]                 // Aliases for various flags
#define cp flag_base[1]
#define glass_cut flag_base[2]
#define gate_oiled flag_base[3]
#define wire_cut flag_base[4]
#define weeds_dead flag_base[5]
#define plant_watered flag_base[6]
#define werewolf_dead flag_base[7]
#define vampire_gone flag_base[8]
#define basilisk_dead flag_base[9]
#define door_open flag_base[10]
#define riddles_solved flag_base[11]     // (not implemented yet)
#define waved flag_base[12]
#define look flag_base[13]
#define game_over flag_base[14]

int gametime=0;                          // Number of turns elapsed
int scr_ct=0;                            // Scroll counter
ubyte v=0,n=0;                           // Verb no, noun no

ubyte visited[NUM_LOC+1]={};             // Flags to hold various information 
ubyte moved[NUM_OBJ+1]={};               // during the game
ubyte location[NUM_OBJ+1]={};

char forest_short[]="Forest";            // Some very common descriptions
char maze_short[]="Maze";
char forest_long[]="You are wandering through a dense forest.";
char maze_long[]="You are wandering through a maze of seemingly identical "
  "passages.";

char *verb[]={"","n","s","e","w","look","score","help","open","wave","cut",
  "light","oil","examine","save","load","off","get","drop","inv","quit",
  "close","climb","jump","cross","spray","wear","remove","water","fill",
  "empty","throw","unload","fire","shoot","kill","break","kick","fuck",
  "verbose","brief","normal","swim"};

ubyte need_obj[NUM_VERB+1]={0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,1,1,1,0,0,1,0,0,0,
  1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0};

char *noun[]={"","torch","","matches","axe","plank","diamond","oil","",
  "hacksaw","weedkiller","","mask","water","bucket","bullet","bracelet","gun",
  "","ring","crucifix","mirror","paper","key","stone","staff","statuette",
  "safe","","emerald","folder","toad","tree","","glass","weeds","wire","gate",
  "werewolf","sign","plant","ogre","door","chasm","stream","vampire",
  "basilisk","all"};

char *obj[]={"","an unlit torch","a blazing torch","a box of matches",
  "an axe","a wooden plank","a diamond","a can of oil","an empty can",
  "a hacksaw","a packet of weedkiller","an empty packet","a gas mask",
  "a bucket of water","an empty bucket","a silver bullet","a silver bracelet",
  "a gun","a gun","a gold ring","a silver crucifix","a mirror",
  "a scrap of paper","a key","a small stone","a long staff",
  "a jade statuette","a locked safe","an open safe","an emerald",
  "a brown folder","a toad","a tree","a fallen tree"};

char *init_obj[]={"","There is an unlit torch lying on the ground.","",
  "You can see a box of matches jammed in a crevice in one branch.",
  "A large axe is lying among the trees.",
  "An old wooden plank is lying in the grass.",
  "The sparkle of a diamond catches your eye from a pile of dust.",
  "There is a can of oil sitting on one shelf.","",
  "There is an old hacksaw sitting on a rickety shelf.",
  "A plastic packet of weedkiller is sitting in the corner.","",
  "A gas mask is hanging from a hook on the wall.","",
  "An empty bucket is lying in the corner.",
  "There appears to be a silver bullet jammed into a crevice in the ground.",
  "A silver bracelet is lying on the ground.",
  "There is a gun lying abandoned in the corner.","",
  "The gleam of gold catches your eye from a gold ring lying in the dust.",
  "There is a silver crucifix hanging by its chain from a hook on the wall.",
  "A large gleaming mirror is leaning against the wall.",
  "You notice a scrap of paper on the ground.",
  "A key is lying in a depression in the ground.",
  "A small stone is sitting in the middle of the path.",
  "A long ornate gilded staff is leaning against the wall.",
  "A jade statuette is sitting on a ledge.",
  "A locked safe is sitting in the corner.",
  "An open safe is sitting in the corner.","There is an emerald in the safe.",
  "There is a brown folder in the safe.",
  "There is an ugly toad sitting in the corner staring balefully at you.",
  "A single large tree is growing in the middle of the path, neatly "
    "blocking your way north.",
  "A fallen tree is lying across the path and extending into the forest."};

char *long_obj[]={"","It's a standard-issue adventure game torch, requiring "
    "a source of fire to light it, and currently unlit.",
  "The torch is blazing, giving off a bright yellow light.",
  "It's a box of matches of the sort that can be bought in any newsagents "
    "or found in most adventure games.",
  "The axe has a long wooden shaft and a sharp, heavy steel blade. An "
    "experimental swing indicates that it's too clumsy to be of much use as "
    "a weapon.",
  "It's a perfectly ordinary long wooden plank. It has no weatherproof "
    "coating of any kind and the edges are a bit rough, not planed very "
    "well.",
  "It's a large, hard, sharp piece of crystalline carbon with a substantial "
    "market value.",
  "The can is small and almost rectangular. The oil is a black sticky "
    "liquid.",
  "It's an empty metal can with black stains around the edge.",
  "A hacksaw is a small saw with a very hard, sharp blade, which can cut "
    "through wood or soft metals.",
  "At the bottom of the label is the legend 2,4,5-T and beside it is the "
    "large black-on-yellow X which chemical companies put on things that "
    "should really carry the skull-and-crossbones poison symbol when they "
    "don't want to frighten their customers. A spray nozzle is on the top of "
    "the packet, and the instructions on the label read: \"To use, type "
    "SPRAY <target>\".",
  "The packet is now empty.",
  "This is a well-made piece of military equipment, standard issue for CBW "
    "(chemical and biological warfare) protection.",
  "The bucket is made of plastic and its contents are made of hydrogen "
    "oxide.",
  "The bucket is made of plastic and its contents are made of nitrogen, "
    "oxygen, carbon dioxide, argon and water vapor, with traces of other "
    "gases.",
  "The bullet is actually a composite of the standard lead (for weight and "
    "cheapness) and silver (for use in horror movies).",
  "The bracelet is highly polished and gleams in the light. It's quite "
    "pretty and more to the point, valuable.",
  "This is a .357 Magnum revolver, capable of blowing away any murderous "
    "speleologists you're likely to encounter (at least if it were loaded).",
  "This is a .357 Magnum revolver, capable of blowing away any murderous "
    "speleologists you're likely to encounter. It contains one bullet.",
  "At first sight the ring seems completely featureless, but closer "
    "examination reveals runic symbols scratched into the metal.",
  "This is an ancient religious symbol. It's also made of solid silver and "
    "you could probably get a good bit of money for it.",
  "You stare into the large gleaming mirror and almost jump through the "
    "ceiling when you see something more hideous than you have ever imagined."
    " You calm down on realizing that it's quite harmless. (OK, only kidding,"
    " actually you're a fine specimen of an adventurer.)",
  "On the paper is written a square root symbol enclosing the number "
    "3169014436.",
  "The function of this key is to open a door. This is its only purpose in "
    "life. As such, it is totally unadorned, and has no interesting features "
    "whatsoever.",
  "It is made of coarse granite. A glance at the scenery reveals that the "
    "hill it comes from is an outcrop of this granite, which survived when "
    "the surrounding sedimentary rocks were eroded away. The rock was "
    "probably formed around the late Cretaceous era, though you cannot be "
    "certain. You resolve to conduct uranium-thorium dating tests on the "
    "stone when you return to civilization.",
  "The staff is long and carved with ornate symbols, all incomprehensible.",
  "The statuette is small and very ugly, but made of green jade and probably "
    "quite valuable.",
  "The safe is locked with a combination lock with a hundred thousand "
    "possible combinations. The walls are made of multiple-layer steel alloy "
    "several inches thick, capable of withstanding drills and explosives and "
    "interspersed with layers of corrosion-resistant metal. You haven't a "
    "hope of breaking into it.",
  "The safe door is open.",
  "You gaze into the green depths of the emerald and see something. It seems "
    "to be a moving shadow. You stare harder and finally make out something "
    "pink and stubby. It seems to be behind the emerald, not within it. You "
    "have discovered one of your fingers.",
  "The folder is full of extremely important and confidential documents of "
    "no interest whatsoever to you.",
  "The toad is peculiarly ugly, warty, garishly coloured and coated with "
    "some sort of glistening slime.",
  "The tree towers above you in all its magnificence. It would be a beautiful"
    " sight were it not for the fact that the trunk is growing right in the "
    "middle of a narrow gap between rocky outcrops and blocking your way "
    "north.",
  "A huge trunk lies across the path. It appears scramble-over-able.",
  "", // Glass-variable appearance
  "", // Weeds-variable appearance
  "", // Wire-variable appearance
  "", // Gate-variable appearance
  "The werewolf is large and dark and hairy and travelling in your direction "
    "at quite a high speed. It seems to have very big sharp teeth.",
  "On closer examination the sign seems to be an award. It's some obscure "
    "acting award on a cheap bronze plaque... the name of the film is SECOND "
    "COUSIN OF DRACULA, and the name of the actor is too tarnished to make "
    "out.",
  "", // Plant-variable appearance
  "The ogre is huge and horribly ugly and extremely fierce looking. That's "
    "all you can see from this distance. It's probably all you want to know.",
  "", // Door-variable appearance
  "The chasm is incredibly deep. It seems to go down forever. It's probably "
    "not really that deep, just that the torchlight can't reach down very "
    "far, but why spoil the fun?",
  "The stream is wide and clear with high banks. It's not flowing very fast. "
    "You catch glimpses of brightly coloured fish swimming in it.",
  "The vampire is tall and wearing the fashionable Dracula-style black cloak."
    " He also has the classical-style long fangs and face as pale as if his "
    "computer had just given him a corrupted-disk error message on a disk "
    "containing two weeks' work.",
  "It's a stone statue of a small harmless-looking lizard."};

ubyte value[]={0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,5,15,0,0,20,15,10,0,0,0,20,5,0,
  0,5,0,0,0,0};

ubyte weight[]={0,3,3,1,6,7,1,3,2,2,6,3,4,6,2,2,2,3,5,1,1,8,1,1,2,7,5,255,255,
  1,3,3,255,255};

ubyte init_loc[]={0,1,0,19,11,14,54,43,0,16,28,0,35,0,16,140,144,44,0,62,56,
  78,21,0,2,90,36,40,0,0,0,65,18,0};

char *lshort[]={"","Clearing","Top of Hill","Steep Narrow Trail",
  "East of Stream","West of Stream","Long Path","Long Path","Clearing","Path",
  "Wide Meadow","Clearing","Narrow Lane","Narrow Lane","Field",
  "Outside Well-House","Well-House","Field","Path","Top of Tree",
  "Cave Entrance","Dusty Room","End of Hall","Great Hall","Wide Corridor",
  "Dark Tunnel","Narrow Passage","Small Cave","Stony Cave","Store Room",
  "Tunnel","Passage","Underground Tunnel","Long Low Tunnel","Damp Cave",
  "Dead End","Cave","Cramped Cave","Gloomy Passage","Underground Tunnel",
  "Dungeon","Stairway","Top of Stairway","Cave","Dead End","Rocky Room",
  "Dank Wet Room","Spacious Cave","Cold Stony Cave","Large Cave",
  "Winding Tunnel","Tunnel","Rocky Alcove","Rocky Tunnel","Store Room",
  "North End of Cavern","Small Room","South End of Cavern","Long Passage",
  "Large Cave","Narrow Passage","Spacious Cave","Dead End","Small Ante-Room",
  "Dirty Narrow Cave","Short Tunnel","Large Cave","Rocky Room","Passage",
  "Dusty Room","Twisting Tunnel","Passage","Cramped Cave","Vampire's Lair",
  "Gloomy Passage","Cave","Dead End","Winding Tunnel","Tunnel",
  "Long High Cave","Winding Tunnel","Echoing Cavern","Tunnel",
  "Winding Passage","Cave","Gloomy Cave","Narrow Passage","Gloomy Tunnel",
  "Series of Rooms","Large Hall","Throne Room",forest_short,forest_short,
  forest_short,forest_short,forest_short,forest_short,forest_short,
  forest_short,forest_short,forest_short,forest_short,forest_short,
  forest_short,forest_short,forest_short,forest_short,forest_short,
  forest_short,forest_short,forest_short,forest_short,forest_short,
  forest_short,forest_short,forest_short,forest_short,forest_short,
  forest_short,forest_short,forest_short,forest_short,forest_short,
  forest_short,forest_short,maze_short,maze_short,maze_short,maze_short,
  maze_short,maze_short,maze_short,maze_short,maze_short,maze_short,
  maze_short,maze_short,maze_short,maze_short,maze_short,maze_short,
  maze_short,maze_short,maze_short,maze_short,"Winding Tunnel","Gloomy Cave"};

char *llong[]={"","This is a small clearing at the top of a hill. Trees "
    "surround you in all directions except west.",
  "You are at the top of a high hill. A trail leads down the hill to the "
    "north and there is a clearing to the east.",
  "You are on a steep narrow trail leading up the north face of a high hill. "
    "The sound of a stream can be heard from below.",
  "You are standing on the east bank of a wide, clear stream. Brightly "
    "coloured fish can be seen swimming in the stream. You are surrounded on "
    "all sides by forest except for a trail which leads up a hill to the "
    "south.",
  "You are standing on the west bank of a wide, rushing stream. You catch "
    "glimpses of some brightly coloured fish in the water. A path leads "
    "west.",
  "You are walking along a long east-west path through the forest. You can "
    "hear the sound of a stream from the east.",
  "This is the western end of a long east-west forest path.",
  "You are in a wide clearing among the trees. The forest surrounds you on "
    "all sides, looking deep and menacing. A single path leads west.",
  "You are on an east-west forest path.",
  "This is a wide, pleasant meadow. A lane leads south and a smaller path "
    "leads east. There is an opening among the trees to the west.",
  "You are in a nondescript forest clearing. More open ground can be seen "
    "to the east.",
  "You are walking along a narrow lane which leads to a meadow to the north "
    "and dwindles into the distance in the south.",
  "You are near the end of a narrow lane which leads north into the forest. "
    "A gate opens onto a field to the west and the lane continues south for "
    "a short distance.",
  "This is a small grassy field in the middle of the woods. A narrow lane "
    "can be reached through a gate to the east.",
  "You are at the end of a lane which continues to the north, and standing "
    "outside a well-house which can be entered to the east.",
  "This is a small, disused well-house of the type familiar to any "
    "experienced adventurer. No water is currently available here; the "
    "building's function is now indicated by the message \"LEAVE YOUR "
    "TREASURE HERE FOR POINTS\" scrawled on one wall. A single door leads "
    "west.",
  "This is a large open field among the woods. You are surrounded on almost "
    "every side by trees. A single path leads north.",
  "You are on a north-south path. To the south lies open ground. To the "
    "north looms a stony hill towards which the path leads.",
  "You are in the branches of a tall tree. The ground looks an unpleasantly "
    "long way away.",
  "You are standing under a high, stony hill which rises to the north. The "
    "hill is too steep to be climbed but directly to the north gapes the "
    "entrance to a dark cave. A path leads south into the forest.",
  "You are in a small dusty room, bare of any ornament. An exit leads east "
    "through a broken pane of glass.",
  "You are at the western end of a long hall. A doorway leads north.",
  "You are in a vast hall which extends a considerable distance to the west. "
    "A doorway opens onto a corridor to the east.",
  "You are at a corner in a wide corridor which connects with a tunnel to "
    "the north and leads into a large chamber to the west.",
  "You are at the start of a dark tunnel. To the south is the exit to the "
    "open air, and passages lead into the blackness to the east and north.",
  "You are in a narrow east-west passage. To the east yawns a vast chasm "
    "which extends down as far as you can see and is probably too wide to "
    "jump over.",
  "You are in a small cave. A tunnel leads off to the north and to the west "
    "gapes a deep chasm.",
  "You are in a stony cave with rough, uneven walls. A single rough-hewn "
    "doorway leads south into a large hall.",
  "This is a small nondescript storeroom opening off the tunnel to the east.",
  "You are in a small east-west tunnel. A wider corridor leads off to the "
    "south.",
  "You are walking along a long gloomy north-south passageway. A narrow "
    "tunnel leads off to the west.",
  "This is a long tunnel which opens into a small cave to the south and "
    "continues for a long distance to the north. A small opening leads east.",
  "You are in a long east-west tunnel with no distinguishing features.",
  "You are in a cold, damp cave. A cramped tunnel goes off to the west.",
  "The tunnel ends abruptly here in a wall of solid rock. This space is "
    "slightly wider than the rest of the tunnel and the walls are somewhat "
    "more finished.",
  "This is a small cave leading off from the slightly larger one to the "
    "west.",
  "You are in a cramped cave. An even smaller cave adjoins this one to the "
    "east. A short tunnel leads off to the south and a longer passageway "
    "leads off to the north. Strands of barbed wire drape from the entrance "
    "to this passage.",
  "You are in a gloomy passage which curves off to the north and opens into "
    "a small cave to the south.",
  "You are in a twisty north-south tunnel. To the south you can see a cave "
    "through an open gate.",
  "You are in a dank dungeon buried deep beneath the surface. The only exit "
    "is via the stairway leading up to the east.",
  "You are on a steep stone stairway which leads deep down into the depths "
    "of the earth to the west and ascends to the east.",
  "You are at the top of a stairway which descends to the west as far as the "
    "eye can make out. An exit leads east.",
  "You are in a small cave. The only exit is south.",
  "The tunnel comes abruptly to a dead end here. The only way you can go is "
    "back to the south.",
  "You are in a large rocky room with rough-hewn walls. Exits lead north, "
    "south and west.",
  "This is a dank, wet chamber. Slime drips from the walls and the sound of "
    "water can be heard in the distance. Exits are north, south and east.",
  "You are in a long spacious cave. There are doorways in the north and "
    "south walls.",
  "You are in a cold stony cave. The only exit leads north.",
  "This is a large cave which adjoins the tunnel to the east.",
  "You are wandering through a narrow winding tunnel. You can go north, east "
    "or west.",
  "You are in a twisting tunnel. Exits are south or west.",
  "You are in a rocky alcove which opens off the main passage to the east.",
  "This is a short east-west tunnel. The words BEWARE OF THE B... are painted"
    " on the wall, trailing off into a long streak.",
  "This is a small store-room which opens off the north end of a large "
    "cavern.",
  "You are standing in the northern end of a huge cavern. Doorways open to "
    "the east, west and north.",
  "You are in a small room connected to a large cavern. The only exit is "
    "west.",
  "You are at the southern end of a vast cavern. An exit leads south.",
  "You are in a long north-south passage. An alcove adjoins the passage to "
    "the west.",
  "You are in a large cave. Passages curve off to the east and west and a "
    "narrow opening leads north.",
  "You are standing in a very narrow straight north-south passage.",
  "You are in a spacious cave. An exit leads north.",
  "The short tunnel leading off from the ante-room comes to a dead end here. "
    "The only exit is west.",
  "You are in a small ante-room with a cave opening to the south and a short "
    "tunnel leading off to the east. A winding passage goes north.",
  "You are in a filthy little cave. An open door leads south and a short "
    "winding passage leads east.",
  "This is a short, obviously unfinished tunnel extending from the larger one"
    " to the east.",
  "You are in a large cave with a high ceiling. A strong wind blows through "
    "here from unseen sources. You can go north, west or east.",
  "You are in a large rocky room. Exits are east and west, or south past a "
    "few scraps of poisoned weed.",
  "You are in a short north-south passage.",
  "You are in a dusty room at the end of the passage. There is a large round "
    "hole in the ceiling.",
  "You are in a twisting tunnel. Exits are north and west.",
  "You are in a north-south tunnel. A doorway opens to the east. You can see "
    "a sign on the wall.",
  "You are in a cramped cave which adjoins the passage to the west.",
  "You are in the vampire's lair. The vampire himself is standing nearby. He "
    "flashes a toothy grin at you.",
  "You are in a gloomy east-west passage. Another passage leads off to the "
    "south.",
  "The passage here opens into a large cave. Exits are east and west and "
    "another doorway leads north.",
  "You are at a dead end and can only go back west.",
  "You are in a narrow winding tunnel and can go east or south.",
  "You are at the end of the tunnel which leads back west.",
  "You are in a long cave with a high roof. Passages lead east and north.",
  "You are in a winding tunnel and can go north or south.",
  "You are in a large cavern in which every sound makes an echo. Tunnels "
    "lie to the north and east.",
  "You are in a tunnel which connects the cavern to the south with another "
    "passage to the north. A cave lies to the east.",
  "You are in a short winding passage. Exits are west and south.",
  "You are in a small cave. The only exit is east.",
  "You are in a gloomy cave. Doorways lead east and west.",
  "You are in a narrow passage. Exits are north and west.",
  "You are in an evil-smelling tunnel. A huge ogre is squatting in the corner"
    " surrounded by piles of old bones from previous adventurers. A leg of "
    "some unidentifiable animal is lying in the corner, half-eaten. You can "
    "go north or south.",
  "You are in one of a series of underground rooms. Doorways lead north and "
    "east and a tunnel exits to the south.",
  "You are in an ancient banquet hall. A doorway leads south.",
  "You are in an ancient throne room, now sadly stripped of practically "
    "everything of value. The ancient throne looms before you and behind is "
    "the door to the west.",
  forest_long,forest_long,forest_long,forest_long,forest_long,forest_long,
  forest_long,forest_long,forest_long,forest_long,forest_long,forest_long,
  forest_long,forest_long,forest_long,forest_long,forest_long,forest_long,
  forest_long,forest_long,forest_long,forest_long,forest_long,forest_long,
  forest_long,forest_long,forest_long,forest_long,forest_long,forest_long,
  forest_long,forest_long,forest_long,forest_long,maze_long,maze_long,
  maze_long,maze_long,maze_long,maze_long,maze_long,maze_long,maze_long,
  maze_long,maze_long,maze_long,maze_long,maze_long,maze_long,maze_long,
  maze_long,maze_long,maze_long,maze_long,
  "You are in a narrow winding tunnel which connects a cave to the north "
    "with a small round passageway to the west.",
  "You are in a gloomy cave with a low roof. The top part of a tall plant "
    "is growing up through a large hole in the floor of the cave and a "
    "passage leads off to the south."};

ubyte exits[][4]={{0,0,0,0},{98,100,99,2},{3,100,1,101},{4,2,123,101},
  {121,3,122,0},{91,100,0,6},{91,124,5,7},{116,105,6,105},{114,105,92,9},
  {113,106,8,10},{112,12,9,11},{108,118,10,118},{10,13,124,120},
  {12,15,106,14},{108,107,13,120},{13,105,16,120},{0,0,0,15},{18,117,121,115},
  {0,17,117,119},{0,0,0,0},{25,18,122,115},{0,0,22,0},{28,0,23,0},{0,0,24,22},
  {30,0,0,23},{31,20,26,0},{0,0,0,25},{32,0,0,0},{0,22,0,0},{0,0,30,0},
  {0,24,31,29},{51,25,0,30},{58,27,33,0},{0,0,34,32},{0,0,0,33},{37,0,0,0},
  {0,0,0,37},{38,35,36,0},{59,0,0,0},{59,34,0,0},{0,0,41,0},{0,0,42,40},
  {0,0,45,41},{0,45,0,0},{0,46,0,0},{43,47,0,42},{44,48,53,0},{45,50,0,0},
  {46,0,0,0},{0,0,50,0},{47,0,51,49},{0,31,0,50},{0,0,58,0},{0,0,55,46},
  {0,55,0,0},{54,57,56,53},{0,0,0,55},{55,58,0,0},{57,32,0,52},{60,0,38,39},
  {0,59,0,0},{63,0,0,0},{0,0,0,63},{64,61,62,0},{0,60,63,0},{0,0,66,0},
  {68,0,67,65},{0,64,70,66},{69,66,0,0},{0,68,0,0},{71,0,0,67},{73,70,72,0},
  {0,0,0,71},{74,71,0,0},{0,73,75,79},{77,0,76,74},{0,0,0,75},{0,75,78,0},
  {0,0,0,77},{80,0,74,0},{81,79,0,0},{82,0,80,0},{83,81,85,0},{0,82,0,84},
  {0,0,83,0},{0,0,86,82},{87,0,0,85},{88,86,0,0},{89,87,90,0},{0,88,0,0},
  {0,0,0,88},{117,6,122,116},{121,5,123,116},{122,98,123,92},{121,99,123,91},
  {117,98,99,93},{116,92,99,94},{122,4,123,96},{98,99,123,117},{98,100,123,3},
  {1,99,98,101},{5,100,2,102},{6,124,101,103},{7,106,102,104},
  {105,124,103,105},{8,124,7,106},{124,107,100,13},{108,106,14,120},
  {119,107,12,118},{110,108,107,120},{118,109,113,120},{112,110,11,118},
  {119,111,115,118},{115,112,114,119},{115,8,91,113},{119,114,17,112},
  {117,91,121,114},{121,92,122,116},{119,120,112,110},{115,111,113,118},
  {118,107,109,119},{117,92,122,116},{121,98,123,94},{122,97,98,117},
  {103,106,100,13},{126,126,145,125},{125,127,125,126},{125,126,128,126},
  {129,127,127,126},{0,128,127,130},{131,128,0,129},{130,129,129,132},
  {127,133,131,130},{132,132,134,131},{126,135,128,133},{0,128,136,134},
  {137,135,126,127},{136,127,128,138},{127,129,137,139},{134,138,140,137},
  {133,141,130,139},{140,142,0,131},{127,140,143,141},{144,142,136,129},
  {141,134,135,143},{146,0,0,125},{0,145,0,0}};

ubyte noun_syn_num[]={N_MATCHES,N_OIL,N_HACKSAW,N_WEEDKILLER,N_WEEDKILLER,
  N_WEEDS,N_WEREWOLF,N_AXE};

char *noun_syn[]={"box","can","saw","packet","pack","plants","wolf","ax"};

ubyte verb_syn_num[]={V_LOOK,V_OPEN,V_CUT,V_CUT,V_LIGHT,V_EXAMINE,V_LOAD,
  V_OFF,V_GET,V_INV,V_INV,V_CLOSE,V_CLOSE,V_KILL,V_KILL,V_BREAK,V_FUCK,
  V_FUCK};

char *verb_syn[]={"l","unlock","chop","scratch","on","read","restore",
  "extinguish","take","i","inventory","shut","lock","hit","attack","smash",
  "shit","bugger"};

int countobj(int);
int listobj(int);
void ogre_death(void);
void init(void);
void get_command(void);

int getch(void)
{
  int ch=ngetchx();
  memset(LCD_MEM+(LCD_HEIGHT-6)*30,0,180);     // Erase status line
  return ch;
}

void printcr(void)
{
  scr_ct++;
  if(scr_ct==(TI89?15:20))
    {
      scr_ct=0;
      DrawStr(LCD_WIDTH-30,LCD_HEIGHT-6,"More...",A_NORMAL);
      getch();
    }
  putchar('\n');
}

void print(char *ptr)
{
  char ch,*temp;
  int b;
  SCR_STATE sstate;
  while(1)
    {
      ch=*ptr++;
      if(!ch) return;
      if(ch=='\n') printcr();
      else if(ch!=' ')
        {
          temp=--ptr;
          b=0;
          while(1)
            {
              ch=*ptr++;
              if(ch==' '||!ch||ch=='\n') break;
              b+=FontCharWidth(ch);
            }
          SaveScrState(&sstate);
          if(sstate.CurX+b>ScrRect->xy.x1) printcr();
          ptr=temp;
          while(1)
            {
              ch=*ptr;
              if(ch==' '||!ch||ch=='\n') break;
              putchar(ch);
              ptr++;
            }
          SaveScrState(&sstate);
          if(sstate.CurX<ScrRect->xy.x1-2&&*ptr) putchar(' ');
        }
    }
}

void getline(void)
{
  SCR_STATE ss;
  int key,i=0;
  buffer[0]=0;
  SaveScrState(&ss);
  do
    {
      printf_xy(ss.CurX,ss.CurY,"%s_   ",buffer);
      key=getch();
      if(key>='A'&&key<='Z') key+=32;
      if(key>=' '&&key<='~'&&i<BUFLEN-1) buffer[i++]=key;
      if(key==257&&i) i--;
      buffer[i]=0;
    } while (key!=13);
  printf_xy(ss.CurX,ss.CurY,"%s   ",buffer);
  scr_ct=0;
  putchar('\n');
}

#define p_ret(s) {print(s); return;}

void status(void)
{
  if(cp==L_WEREWOLF)
    {
      if(werewolf_dead==1)
	{
          game_over=TRUE;
          p_ret("The werewolf reaches you and rips your throat out before "
            "gorging itself on your flesh and blood.\n");
	}
      if(!werewolf_dead)
	{
          print("A werewolf rises from the corner and sees you. It starts "
            "sprinting towards you at amazing speed, and very rapidly comes "
            "close enough to provide a good view of a mouthful of slavering "
            "fangs.\n");
          werewolf_dead=1;
	}
    }
  if(cp==L_VAMPIRE&&location[N_CRUCIFIX]!=CARRIED&&location[N_CRUCIFIX]!=WORN)
    {
      game_over=TRUE;
      p_ret("The vampire seizes you and sinks his fangs into your neck. Your "
        "struggles are fruitless against the strength of the undead, not to "
        "mention the fact that this vampire has been doing a lot of working "
        "out with the weights to while away the long years of waiting for "
        "victims. The slurping sound of your blood being sucked out sounds "
        "loud in your ears. Eventually your heart stops from lack of blood, "
        "but the vampire gives you cardio-pulmonary resuscitation to extract "
        "the last few drops. Finally, you die.\n");
    }
  if(cp==L_DRAUGHTS)
    {
      if(rand()%4) print("Your torch flickers in the draught.\n");
      else
	{
          print("A particularly strong gust of wind blows out your torch. "
            "You are instantly plunged into Stygian gloom.\n");
          location[N_TORCH_UNLIT]=location[N_TORCH_LIT];
          location[N_TORCH_LIT]=NOWHERE;
	}
    }
}

void desc_loc(void)
{
  int i;
  if(cp==L_STREAM_WEST&&location[N_PLANK]==L_STREAM_EAST)
    location[N_PLANK]=L_STREAM_WEST;
  if(cp==L_STREAM_EAST&&location[N_PLANK]==L_STREAM_WEST)
    location[N_PLANK]=L_STREAM_EAST;
  if(!look&&location[N_TORCH_LIT]!=CARRIED&&location[N_TORCH_LIT]!=cp
    &&(rand()%5)==0&&((cp>20&&cp<91)||cp>124))
      {
        game_over=TRUE;
        p_ret("You have stumbled in the dark, fallen into a pot-hole and "
          "broken your neck.\n");
      }
  if(location[N_TORCH_LIT]!=CARRIED&&location[N_TORCH_LIT]!=cp&&
    ((cp>20&&cp<91)||cp>124))
      {
        print("It's too dark to see anything clearly.\n");
	goto SKIP_SCENERY;
      }
  putchar('['); print(lshort[cp]); print("]\n");
  if(((char)mode-(char)visited[cp]>0)||look)
    // A function of what description mode is active and have we been
    // here before in this game
    {
      print(llong[cp]); printcr();
    }
  for(i=1;i<=NUM_OBJ;i++)
    {
      if(location[i]==cp&&!moved[i])
	{
          print(init_obj[i]); printcr();
	}
    }
  if(countobj(cp))
    {
      print("You can also see ");
      listobj(cp);
    }
  if(cp<91||cp>144) visited[cp]=TRUE;     // Not in forest or maze
  if(location[N_PLANK]==cp)
    {
      if(cp==L_STREAM_WEST||cp==L_STREAM_EAST)
        print("The plank is lying across the stream.\n");
      if (cp==L_CHASM_WEST||cp==L_CHASM_EAST)
        print("The plank is bridging the chasm.\n");
    }
  if(cp==L_GLASS)
    {
      if(glass_cut)
        print("A few jagged glass fragments surround the passage to the "
          "west.\n");
      else
        print("The passage to the west is sealed off by a large sheet of "
          "darkened glass.\n");
    }
  if(cp==L_GATE)
    {
      if(gate_oiled==2)
        print("There is a passage to the north through an open gate.\n");
      else
        print("A passage to the north is blocked off by a rusty old gate.\n");
    }
  if(cp==L_WIRE)
    {
      if(wire_cut)
        print("A passage leads south through some strands of barbed wire.\n");
      else
        print("A passage to the south is blocked by a tangle of barbed "
          "wire.\n");
    }
  if(cp==L_BASILISK)
    {
      if(basilisk_dead)
        print("There is a stone statue of a basilisk here.\n");
      else
	{
          print("You hear movement in the shadows. Suddenly a basilisk comes "
            "rushing across the floor at you!\n");
          if(location[N_MIRROR]==CARRIED)
	    {
              print("The basilisk's deadly gaze tracks up towards your face. "
                "You are frozen with horror for a second, as if in "
                "anticipation of your fate. Before you can do anything, the "
                "creature looks into the mirror you are holding. There is a "
                "single screech of horror, abruptly cut off. Its outline "
                "appears to flicker for a moment, and its movement towards "
                "you comes to a halt. You sigh with relief as the ossified "
                "corpse obviously presents no further danger.\n");
              basilisk_dead=TRUE;
	    }
          else
	    {
              print("Before you can react, the basilisk's deadly gaze tracks "
                "across your face. The last thought that flashes through your"
                " brain before it stops thinking forever is that the thing "
                "looks almost cute from this angle.\n");
              game_over=TRUE;
	    }
	}
    }
  if(cp==L_WEEDS)
    {
      if(weeds_dead)
        print("The blackened remains of some weeds lie on the floor of the "
          "passage to the north.\n");
      else
        print("The passage north is blocked by a dense growth of weeds.\n");
    }
  if(cp==L_PLANT)
    {
      if(plant_watered)
        print("A tall plant is growing up through the hole.\n");
      else
        print("A smallish plant is growing under the hole.\n");
    }
  if(cp==L_DOOR)
    {
      if(door_open==1) print("An open door leads north.\n");
      else print("The way north is blocked by a locked door.\n");
    }
SKIP_SCENERY:
  if(cp==L_OGRE)
    {
      if(location[N_RING]==WORN)
        print("The ogre cannot see you while you are wearing the ring of "
          "invisibility.\n");
      else
	{
          print("The ogre sees you.\n");
          ogre_death();
          return;
	}
    }
  if(cp==L_VAMPIRE&&(location[N_CRUCIFIX]==WORN||
    location[N_CRUCIFIX]==CARRIED))
      print("The vampire glances back and forth between the crucifix you are "
        "carrying and your jugular vein, but seems to be of the opinion that "
        "discretion is the better part of valour.\n");
  look=0;
}

void get(int o)
{
  int i,tot_weight;
  if(location[o]==CARRIED||location[o]==WORN) p_ret("You already have it!\n");
  if(weight[o]>MAXWEIGHT) p_ret("It's too heavy to carry.\n");
  tot_weight=0;
  for(i=1;i<=NUM_OBJ;i++)
    if(location[i]==CARRIED||location[i]==WORN) tot_weight+=weight[i];
  if(tot_weight+weight[o]>MAXWEIGHT)
    p_ret("You're carrying too much already.\n");
  location[o]=CARRIED;
  if(o==N_STONE&&moved[N_STONE]==FALSE)
    {
      location[N_KEY]=cp;
      moved[N_STONE]=TRUE;
      p_ret("You pick up the stone and notice a key that had been hidden "
        "underneath it.\n");
    }
  if(o==N_TOAD)
    {
      game_over=TRUE;
      p_ret("This species of toad secretes a powerful nerve poison as a "
        "defence against predators and acquisitive adventurers. As you touch "
        "its skin you absorb some of the poison. It works very rapidly. You "
        "die within minutes.\n");
    }
  print("Taken.\n");
  moved[o]=TRUE;
}

void drop(int o)
{
  if(location[o]==WORN) p_ret("You'll have to remove it first.\n");
  if(location[o]!=CARRIED) p_ret("You're not holding that.\n");
  print("Dropped.\n");
  location[o]=cp;
  if(o==N_PLANK)
    {
      if(cp==L_STREAM_WEST||cp==L_STREAM_EAST)
        print("It just extends across the width of the stream.\n");
      if(cp==L_CHASM_WEST||cp==L_CHASM_EAST)
        print("It just extends across the width of the chasm.\n");
    }
  if(o==N_RING&&cp==L_OGRE)
    {
      print("Unfortunately, the ogre notices you now that you are no longer "
        "wearing the magic ring.\n");
      ogre_death();
    }
}

int countobj(int l)
{
  int i,objects;
  objects=0;
  for(i=1;i<=NUM_OBJ;i++)
    if(location[i]==l&&moved[i]) objects++;
  return objects;
}

int listobj(int l)
{
  int i,objects,returnobjects;
  char str[1000];
  returnobjects=objects=countobj(l);
  for(i=1;i<=NUM_OBJ;i++)
    {
      if(location[i]==l&&moved[i])
	{
          strcpy(str,obj[i]);
          objects--;
          if(objects>1) strcat(str,", ");
          if(objects==1) strcat(str," and ");
          if(!objects) strcat(str,".\n");
          print(str);
	}
    }
  return returnobjects;
}

void spray_death(void)
{
  location[N_WEEDKILLER]=NOWHERE;
  location[N_PACKET]=CARRIED;
  if(location[N_MASK]==WORN)
    print("Clouds of toxic chemical float around your head. You sniff the "
      "air anxiously, but the gas mask filters it all out.\n");
  else
    {
      print("You have been unconsciously holding your breath while the "
        "weedkiller does its work, but some of the stuff gets into your "
        "lungs anyway. You begin to feel dizzy and sick. Suddenly an "
        "invisible hand seems to clamp down on your chest. Your muscles are "
        "paralyzed and you are unable to breathe. You die by slow "
        "suffocation.\n");
      game_over=TRUE;
    }
}

void ogre_death(void)
{
  print("The ogre leaps upon you with a bellow of rage. You are savagely "
    "butchered in a manner far too hideous to be explicitly described in a "
    "family game. Your remains are left for a couple of weeks to become "
    "nicely rotten, then eaten with relish.\n");
  game_over=TRUE;
}

int check_werewolf(void)
{
  if(cp==L_WEREWOLF&&werewolf_dead==1)
    {
      print("You start to run for the exit, but the werewolf is much faster "
        "than you are. It reaches you and rips your throat out, then gorges "
        "itself on your flesh and blood.\n");
      game_over=TRUE;
      return TRUE;
    }
  return FALSE;
}

int yesno(void)
{
  int c;
  do
    c=getch();
  while(c!='y'&&c!='Y'&&c!='n'&&c!='N');
  putchar(c); printcr();
  scr_ct=0;
  return(c=='y'||c=='Y');
}

void do_command(void)
{
  int i,j,k,failed;
  SYM_ENTRY *sym_entry;
  SCR_STATE ss;
  ubyte *fptr,*bptr;
  char str[100];
  i=j=k=0;
  gametime++;
  switch(v)
    {
      case V_NORTH:
        if(check_werewolf()) return;
        if(exits[cp][0])
          {
            cp=exits[cp][0];
            desc_loc();
            return;
          }
        if(cp==L_TREE)
          {
            if(location[N_TREE]==NOWHERE)
              {
                cp=20;
                desc_loc();
                return;
              }
            p_ret("The tree blocks your way.\n");
          }
        if(cp==L_WEEDS)
          {
            if(weeds_dead)
              {
                cp=67;
                desc_loc();
                return;
              }
            p_ret("The weeds form a tangle too dense to struggle through.\n");
          }
        if(cp==L_GATE)
          {
            if(gate_oiled==2)
              {
                cp=39;
                desc_loc();
                return;
              }
            p_ret("You can't walk through the closed gate.\n");
          }
        if(cp==L_DOOR)
          {
            if(door_open==1)
              {
                cp=64;
                desc_loc();
                return;
              }
            p_ret("The locked door bars your way.\n");
          }
        p_ret("You can't go that way.\n");
      case V_SOUTH:
        if(check_werewolf()) return;
        if(exits[cp][1])
          {
            cp=exits[cp][1];
            desc_loc();
            return;
          }
        if(cp==L_WIRE)
          {
            if(wire_cut)
              {
                cp=37;
                desc_loc();
                return;
              }
            p_ret("The barbed wire blocks your way.\n");
          }
        p_ret("You can't go that way.\n");
      case V_EAST:
EAST:   if(exits[cp][2])
          {
            cp=exits[cp][2];
            desc_loc();
            return;
          }
        if(cp==L_STREAM_WEST)
          {
            if(location[N_PLANK]==L_STREAM_WEST)
              {
                cp=L_STREAM_EAST;
                desc_loc();
                return;
              }
            p_ret("The stream blocks your path.\n");
          }
        if(cp==L_CHASM_WEST)
          {
            if(location[N_PLANK]==L_CHASM_WEST)
              {
                location[N_PLANK]=cp=L_CHASM_EAST;
                desc_loc();
                return;
              }
            p_ret("The chasm blocks your way.\n");
          }
        p_ret("You can't go that way.\n");
      case V_WEST:
WEST:   if(exits[cp][3])
          {
            cp=exits[cp][3];
            desc_loc();
            return;
          }
        if(cp==L_STREAM_EAST)
          {
            if(location[N_PLANK]==L_STREAM_EAST)
              {
                cp=L_STREAM_WEST;
                desc_loc();
                return;
              }
            p_ret("The stream blocks your path.\n");
          }
        if(cp==L_CHASM_EAST)
          {
            if(location[N_PLANK]==L_CHASM_EAST)
              {
                location[N_PLANK]=cp=L_CHASM_WEST;
                desc_loc();
                return;
              }
            p_ret("The chasm blocks your way.\n");
          }
        if(cp==L_GLASS)
          {
            if(glass_cut)
              {
                cp=L_BEYOND_GLASS;
                desc_loc();
                return;
              }
            p_ret("The glass blocks your way.\n");
          }
        p_ret("You can't go that way.\n");
      case V_LOOK:
        look=1;
        desc_loc();
        return;
      case V_SCORE:
        for(i=1;i<=NUM_OBJ;i++)
          if(location[i]==L_DROPZONE) j+=value[i];
        sprintf(str,"You have scored %u points in %u moves.\n",j,gametime);
        print(str);
        if(j==100)
          {
            print("Congratulations! You have scored maximum points and "
              "completed the game!\n");
            game_over=TRUE;
          }
        return;
      case V_HELP:
        p_ret("Sorry, but it is not my policy to give help in adventure games"
          " as I believe that it spoils the fun of solving the problems "
          "yourself (this despite the fact that I usually have to ask people "
          "for help in most of the problems in adventure games I play, but "
          "this game isn't too difficult anyway... I suspect the real reason "
          "is that I'm too bloody lazy to put in a proper help facility). C "
          "source code is included, so you can look at it if you're really "
          "desperate (this discourages casual cheating).\n");
      case V_OPEN:
        if(n==N_SAFE_OPEN) p_ret("It's already open.\n");
        if(n==N_FOLDER) goto EXAMINE;
        if(n==N_GATE)
          {
            if(gate_oiled==2) p_ret("It's already open.\n");
            if(!gate_oiled) p_ret("It's too stiff to open.\n");
            gate_oiled=2;
            p_ret("The gate opens with a noisy squeal.\n");
          }
        if(n==N_DOOR)
          {
            if(door_open==1) p_ret("It's already open.\n");
            if(location[N_KEY]!=CARRIED)
              p_ret("It's locked and you haven't got the key.\n");
            if(door_open==2)
              p_ret("The door is quite badly damaged and the keyhole is "
                "smashed. You can't open it now.\n");
            door_open=TRUE;
            p_ret("You insert the key in the lock and turn. Nothing happens. "
              "You jiggle the key around and exert more pressure. Slowly it "
              "turns. The door creaks open.\n");
          }
        if(n==N_SAFE_LOCKED)
          {
            print("Enter combination number: ");
            getline();
            if(strcmp(buffer,"56294"))
              p_ret("You turn the dials to that number but hear no click. "
                "You pull on the door anyway but it doesn't open.\n");
            print("You turn the dial to that number and hear a loud click. "
              "The door swings open.\n");
            location[N_SAFE_LOCKED]=NOWHERE;
            location[N_SAFE_OPEN]=L_SAFE;
            if(moved[N_EMERALD]==FALSE)
              {
                location[N_EMERALD]=L_SAFE;
                print("You see a beautiful emerald inside.\n");
              }
            if(moved[N_FOLDER]==FALSE)
              {
                location[N_FOLDER]=L_SAFE;
                print("You see a plain brown folder inside.\n");
              }
            return;
          }
        p_ret("There's no point in trying to open that.\n");
      case V_WAVE:
        if(waved)
          p_ret("The damage has been done. There's no way you can undo it "
            "now.\n");
        waved=TRUE;
        p_ret("You start to wave vigorously. No effect is immediately "
          "apparent. However, you are generating waves of gravitational "
          "energy which radiate in all directions at the speed of light. "
          "These waves are incredibly weak but as luck would have it they "
          "happen to be at just the right frequency to resonate with the "
          "detector apparatus of an alien interstellar probe ship on the "
          "edge of the Solar System. The ship immediately sends a message to "
          "its home planet to the effect that intelligent life has been "
          "detected on Earth. Within days a mighty battle fleet is on its "
          "way with the mission of utterly destroying this planet. It will "
          "arrive within a hundred years. You have sealed the fate of the "
          "human race.\n");
      case V_CUT:
CUT:    if(n==N_FALLEN_TREE) p_ret("I think your point has been made.\n");
        if(n==N_TREE)
          {
            if(location[N_AXE]!=CARRIED)
              p_ret("Now is the time to put all those hours playing Karate "
                "Kid 2 to some good use! With a blood-curdling yell, you chop"
                " the base of the tree with your hand. The tree infuriatingly"
                " fails to fall down.\n");
            location[N_TREE]=NOWHERE;
            location[N_FALLEN_TREE]=L_TREE;
            p_ret("You swing the axe at the base of the tree trunk again and "
              "again, cutting deeper and deeper into the wood. Finally the "
              "tree topples with a mighty roar, and you have just illegally "
              "vandalized a natural area. However, you can now go north.\n");
          }
        if(n==N_GLASS)
          {
            if(glass_cut) p_ret("There's no need to overdo it!\n");
            if(location[N_DIAMOND]!=CARRIED)
              {
                if(location[N_HACKSAW]==CARRIED)
                  p_ret("You can't get any purchase on the glass with the "
                    "hacksaw.\n");
                if(location[N_AXE]==CARRIED)
                  p_ret("You swing the axe at the glass! It strikes with a "
                    "dull thud but the glass is a special shatterproof "
                    "variety and the blade merely bounces off.\n");
                if(location[N_EMERALD]==CARRIED)
                  p_ret("You try scraping the emerald against the glass, but "
                    "it doesn't seem to be quite hard and sharp enough.\n");
                p_ret("You've nothing sharp enough.\n");
              }
            glass_cut=TRUE;
            p_ret("You press the point of the diamond against the glass hard "
              "and cut a long groove in it. You extend the groove until it "
              "forms a complete circle. A couple of hard kicks suffice to "
              "smash the weakened glass out. The way west is now clear.\n");
          }
        if(n==N_WIRE)
          {
            if(wire_cut) p_ret("You've already done that.\n");
            if(location[N_HACKSAW]!=CARRIED)
              {
                if(location[N_AXE]==CARRIED)
                  p_ret("You swing the axe at the tangle of wire but the "
                    "wire bends and the blade gets tangled in it. You spend "
                    "half a minute removing it.\n");
                p_ret("You've no suitable cutting implement.\n");
              }
            wire_cut=TRUE;
            p_ret("You sit down and diligently start sawing away at the "
              "strands of wire. A long time later, you cut through the last "
              "strand. Your way south is now clear.\n");
          }
        if(n==N_DOOR)
          {
            if(door_open==1)
              p_ret("The door no longer presents any obstacle.\n");
            if(location[N_AXE]!=CARRIED)
              {
                if(location[N_HACKSAW]==CARRIED)
                  p_ret("You can't get the hacksaw blade into the door.\n");
                p_ret("You've no tool capable of chopping down a door.\n");
              }
            if(door_open==2)
              {
                door_open=1;
                p_ret("You attack the door again with the axe. After much "
                  "effort you succeed in smashing it open.\n");
              }
            door_open=2;
            p_ret("You attack the door with the axe. You seem to have done "
              "some damage, but the door still stands in your way.\n");
          }
        if(n==N_PLANT)
          {
            if(location[N_AXE]!=CARRIED)
              p_ret("You're not carrying an axe to chop it down with.\n");
            p_ret("You swing the axe at the plant. Surprisingly, the blade "
              "merely bounces off the soft-looking stem.\n");
          }
        p_ret("There's no point in trying to damage that.\n");
      case V_LIGHT:
        if(n==N_MATCHES)
          p_ret("You light a match. It burns for a short time before going "
            "out.\n");
        if(n==N_TORCH_LIT) p_ret("It's already alight.\n");
        if(location[N_MATCHES]!=CARRIED)
          p_ret("You've no matches to light it with.\n");
        if(n!=N_TORCH_UNLIT)
          p_ret("There's no point in trying to light that.\n");
        location[N_TORCH_LIT]=location[N_TORCH_UNLIT];
        location[N_TORCH_UNLIT]=NOWHERE;
        p_ret("You strike a match and set the torch alight. It burns with a "
          "bright yellow flame.\n");
      case V_OIL:
        if(location[N_CAN]==CARRIED) p_ret("The oil's all been used up.\n");
        if(location[N_OIL]!=CARRIED)
          p_ret("You've nothing to oil it with.\n");
        if(n!=N_GATE)
          p_ret("There's no point in trying to lubricate that.\n");
        gate_oiled=1;
        location[N_CAN]=CARRIED;
        location[N_OIL]=NOWHERE;
        p_ret("You apply the oil to the rusty hinges. They ought to move "
          "more easily now.\n");
      case V_EXAMINE:
EXAMINE:if(n==N_GLASS)
          {
            if(glass_cut)
              p_ret("A few jagged shards of glass still line the "
                "corridor.\n");
            p_ret("A solid sheet of glass blocks the corridor.\n");
          }
        if(n==N_WEEDS)
          {
            if(weeds_dead) p_ret("Dead scraps of weed litter the floor.\n");
            p_ret("The way north is blocked by a dense tangle of weeds.\n");
          }
        if(n==N_WIRE)
          {
            if(wire_cut)
              p_ret("A gap opens to the south through the barbed wire.\n");
            p_ret("Strands of barbed wire stretch across the corridor, "
              "blocking the way south.\n");
          }
        if(n==N_GATE)
          {
            if(gate_oiled==2) p_ret("The gate is open.\n");
            p_ret("The gate is closed.\n");
          }
        if(n==N_PLANT)
          {
            if(plant_watered)
              p_ret("The plant now grows up to the ceiling and the tip "
                "extends through the hole.\n");
            p_ret("The plant is small and nondescript-looking. It looks like "
              "a generic plant rather than any particular species you are "
              "familiar with.\n");
          }
        if(n==N_DOOR)
          {
            if(!door_open) p_ret("The door is locked.\n");
            if(door_open==1) p_ret("The door is open.\n");
            p_ret("The door is locked. It seems damaged, as if some vandal "
              "had attacked it with an axe.\n");
          }
        print(long_obj[n]); printcr();
        return;
      case V_SAVE:
        print("Please enter filename\n (default is ");
        print(filename+1);
        print("): ");
        getline();
        putchar('\n');
        if(buffer[0]) strcpy(filename+1,buffer);
        failed=0;
        SaveScrState(&ss);
        sym_entry=DerefSym(SymAdd(filename+1+strlen(filename+1)));
        if(!sym_entry) failed=1;
        RestoreScrState(&ss);   // SymAdd may open CreateFolder dialog, which
        ST_busy(ST_CLEAR);      // may change system font and print position!
        if(!failed)
          {
            sym_entry->handle=HeapAlloc(100+2*NUM_OBJ+NUM_LOC+FLAGS_SIZE);
            bptr=HeapDeref(sym_entry->handle);
            if(!sym_entry->handle) failed=1;
          }
        if(failed)
          {
            SymDel(filename+1+strlen(filename+1));
            p_ret("Unable to create file.\n");
          }
        fptr=bptr+2;
        memcpy(fptr,location+1,NUM_OBJ); fptr+=NUM_OBJ;
        memcpy(fptr,moved+1,NUM_OBJ); fptr+=NUM_OBJ;
        memcpy(fptr,visited+1,NUM_LOC); fptr+=NUM_LOC;
        memcpy(fptr,flag_base,FLAGS_SIZE); fptr+=FLAGS_SIZE;
        memcpy(fptr,&gametime,sizeof(gametime)); fptr+=sizeof(gametime);
        memcpy(fptr,signature,9); fptr+=9;
        *(int*)bptr=fptr-bptr-2;
        p_ret("Save completed.\n");
      case V_LOAD:
        if(!n)
          {
            print("Please enter filename\n (default is ");
            print(filename+1);
            print("): ");
            getline();
            putchar('\n');
            if(buffer[0]) strcpy(filename+1,buffer);
            sym_entry=SymFindPtr(filename+1+strlen(filename+1),0);
            if(!sym_entry) p_ret("Unable to open file.\n");
            fptr=HeapDeref(sym_entry->handle)+2;
            memcpy(location+1,fptr,NUM_OBJ); fptr+=NUM_OBJ;
            memcpy(moved+1,fptr,NUM_OBJ); fptr+=NUM_OBJ;
            memcpy(visited+1,fptr,NUM_LOC); fptr+=NUM_LOC;
            memcpy(flag_base,fptr,FLAGS_SIZE); fptr+=FLAGS_SIZE;
            memcpy(&gametime,fptr,sizeof(gametime)); fptr+=sizeof(gametime);
            if(memcmp(fptr,signature,9))
              {
                print("Bad or corrupted file.\nPress any key to exit.");
                getch();
                exit(0);
              }
            desc_loc();
            return;
          }
        if(n==N_GUN_LOADED) p_ret("It's already loaded.\n");
        if(n!=N_GUN_UNLOADED) p_ret("You can't load that.\n");
        if(location[N_BULLET]!=CARRIED)
          p_ret("You don't have any ammunition.\n");
        location[N_GUN_LOADED]=location[N_GUN_UNLOADED];
        location[N_GUN_UNLOADED]=location[N_BULLET]=NOWHERE;
        p_ret("The gun is now loaded with one bullet.\n");
      case V_OFF:
        if(n!=N_TORCH_LIT) p_ret("That's not alight!\n");
        location[N_TORCH_UNLIT]=location[N_TORCH_LIT];
        location[N_TORCH_LIT]=NOWHERE;
        p_ret("You snuff out the torch.\n");
      case V_GET:
        if(n==N_ALL)
          {
            for(i=1;i<=NUM_OBJ;i++)
              {
                if(location[i]==cp)
                  {
                    print(obj[i]);
                    print(": ");
                    get(i);
                    k=TRUE;
                  }
                if(game_over) return;  // Trying to get some things is fatal
              }
            if(!k) print("There's nothing here to get!\n");
            return;
          }
        if(n>33) p_ret("You can't carry that.\n");
        get(n);
        return;
      case V_DROP:
        if(n==N_ALL)
          {
            for(i=1;i<=NUM_OBJ;i++)
              {
                if(location[i]==CARRIED)
                  {
                    print(obj[i]);
                    print(": ");
                    drop(i);
                    k=TRUE;
                  }
                if(game_over) return;
              }
            if(!k) print("You have nothing to drop!\n");
            return;
          }
        drop(n);
        return;
      case V_INV:
        print("You are carrying ");
        if(!listobj(CARRIED)) print("nothing.\n");
        if(countobj(WORN))
          {
            print("You are wearing ");
            listobj(WORN);
          }
        return;
      case V_QUIT:
        print("Are you sure you want to quit now? (Y/N) ");
        game_over=yesno();
        return;
      case V_CLOSE:
        if(n==N_SAFE_OPEN)
          {            
            if(!moved[N_EMERALD]) location[N_EMERALD]=NOWHERE;
            if(!moved[N_FOLDER]) location[N_FOLDER]=NOWHERE;
            location[N_SAFE_LOCKED]=L_SAFE;
            location[N_SAFE_OPEN]=NOWHERE;
            p_ret("You swing the door of the safe shut.\n");
          }
        if(n==N_SAFE_LOCKED) p_ret("It's already shut.\n");
        if(n==N_GATE)
          {
            if(gate_oiled!=2) p_ret("It's already closed.\n");
            gate_oiled=1;
            p_ret("The gate swings shut.\n");
          }
        if(n==N_DOOR)
          {
            if(door_open!=1) p_ret("It isn't open.\n");
            p_ret("The door is stuck. It won't close.\n");
          }
        p_ret("You can't close that.\n");
      case V_CLIMB:
        if(cp==L_TREE)
          {
            if(location[N_TREE]==NOWHERE)
              p_ret("You can't climb a chopped-down tree!\n");
            print("You climb up the tree.\n\n");
            cp=L_TREETOP;
            desc_loc();
            return;
          }
        if(cp==L_TREETOP)
          {
            print("You scramble back down the tree.\n\n");
            cp=L_TREE;
            desc_loc();
            return;
          }
        if(cp==L_PLANT)
          {
            if(!plant_watered) p_ret("The plant's too small to climb.\n");
            print("You climb up the plant and through the hole in the "
              "ceiling.\n\n");
            cp=L_TOP_PLANT;
            desc_loc();
            return;
          }
        if(cp==L_TOP_PLANT)
          {
            print("You slide down the plant stalk.\n\n");
            cp=L_PLANT;
            desc_loc();
            return;
          }
        p_ret("There's nothing around here worth climbing.\n");
      case V_JUMP:
        if(cp==L_STREAM_WEST||cp==L_STREAM_EAST)
          {
            game_over=TRUE;
            p_ret("You move back a few steps and take a running jump at the "
              "stream. Despite your valiant attempt, you don't really come "
              "anywhere near jumping across it and merely fall into the "
              "water. This wouldn't matter so much were it not for the fact "
              "that the stream is full of piranha fish (the brightly "
              "coloured fish you saw swimming in it). You can probably "
              "imagine the rest. Anyway, suffice it to say that you end up "
              "fairly dead.\n");
          }
        if(cp==L_CHASM_WEST||cp==L_CHASM_EAST)
          {
            game_over=TRUE;
            p_ret("You walk back a reasonable distance and make a valiant "
              "attempt to jump across the chasm from a running start. "
              "Unfortunately you don't quite make it. Actually, I'm being "
              "charitable here. To tell the truth, you don't come anywhere "
              "near making it. You plummet into the depths. You forget to "
              "count the seconds as you fall and are thus unaware of the "
              "precise depth of the chasm but it seems to be deep enough "
              "to kill you anyway.\n");
          }
        if(cp==L_TREETOP)
          {
            game_over=TRUE;
            p_ret("You close your eyes, spread your arms wide and leap "
              "from the branches of the tree. There is a glorious moment "
              "of free fall with the air rushing past at a tremendous "
              "speed, then you strike the ground head first. The impact "
              "bends your head back, breaking your neck and severing your "
              "spinal cord. Death is instant and painless.\n");
          }
        p_ret("Whee! Such fun!\n");
      case V_CROSS:
        if(cp==L_CHASM_WEST||cp==L_STREAM_WEST) goto EAST;
        if(cp==L_CHASM_EAST||cp==L_STREAM_EAST) goto WEST;
        p_ret("Just type a direction please.\n");
      case V_SPRAY:
SPRAY:  if(location[N_PACKET]==CARRIED)
          p_ret("The weedkiller's all used up.\n");
        if(location[N_WEEDKILLER]!=CARRIED)
          p_ret("You've nothing to use on it.\n");
        if(n==N_WEEDS)
          {
            print("You aim the nozzle at the weeds and press the trigger. "
              "A jet of chemical squirts out, disperses into a mist of "
              "droplets and settles over the plants. The effect is almost "
              "immediate. The plants turn brown, shrivel up and die. Only "
              "a few tattered remnants still lie in the passage. Your way "
              "north is now clear.\n");
            spray_death();
            weeds_dead=TRUE;
            return;
          }
        p_ret("That isn't a weed! Why waste weedkiller on it?\n");
      case V_WEAR:
        if(location[n]==WORN) p_ret("You're already wearing it!\n");
        if(location[n]!=CARRIED) p_ret("You're not holding it.\n");
        if(n!=N_MASK&&n!=N_RING&&n!=N_CRUCIFIX)
          p_ret("That cannot be worn.\n");
        location[n]=WORN;
        print("Worn.\n");
        if(n==N_RING)
          print("This is a magic ring, and confers a degree of invisibility, "
            "at least to something with low intelligence and bad vision.\n");
        return;
      case V_REMOVE:
        if(location[n]!=WORN) p_ret("You're not wearing it.\n");
        print("Removed.\n");
        location[n]=CARRIED;
        if(n==N_RING&&cp==L_OGRE)
          {
            print("Unfortunately, the ogre notices you now that you are "
              "no longer wearing the magic ring.\n");
            ogre_death();
          }
        return;
      case V_WATER:
        if(location[N_WATER]!=CARRIED)
          p_ret("You're not carrying any water.\n");
        if(n!=N_PLANT) p_ret("It isn't in need of moisture.\n");
        print("You pour the water over the leaves of the plant and the "
          "ground at its base.\n");
        if(!plant_watered)
          {
            print("Nothing seems to happen for a short time. Then, "
              "suddenly, the plant starts to expand. Shrivelled leaves "
              "unfurl and the stem creeps higher and higher. New branches "
              "grow out from the stem and sprout more leaves. Finally the "
              "growth stops but by now the plant has grown up through the "
              "hole in the roof.\n");
            plant_watered=TRUE;
          }
        else
          print("Nothing happens. I think the plant already has enough "
            "water.\n");
        location[N_WATER]=NOWHERE;
        location[N_BUCKET]=CARRIED;
        return;
      case V_FILL:
        if(location[n]!=CARRIED&&location[n]!=WORN)
          p_ret("You haven't got it.\n");
        if(n==N_WATER||n==N_WEEDKILLER||n==N_OIL)
          p_ret("It's already full.\n");
        if(cp!=L_STREAM_WEST&&cp!=L_STREAM_EAST)
          p_ret("There's nothing around here to fill it with.\n");
        if(n==N_PACKET||n==N_CAN)
          p_ret("You dip it into the stream in an attempt to fill it but "
            "the opening is too narrow and it's the wrong shape. Your "
            "attempts are unsuccessful.\n");
        if(n!=N_BUCKET) p_ret("You can't fill that!\n");
        location[N_BUCKET]=NOWHERE;
        location[N_WATER]=CARRIED;
        p_ret("You dip the bucket into the stream and fill it with water.\n");
      case V_EMPTY:
        if(location[n]!=CARRIED&&location[n]!=WORN)
          p_ret("You're not holding it.\n");
        if(n==N_CAN||n==N_PACKET||n==N_BUCKET) p_ret("It's already empty.\n");
        if(n==N_OIL)
          {
            location[N_OIL]=NOWHERE;
            location[N_CAN]=CARRIED;
            p_ret("You open the can and pour the oil out onto the ground. It "
              "slowly trickles away in black rivulets.\n");
          }
        if(n==N_WEEDKILLER)
          {
            location[N_WEEDKILLER]=NOWHERE;
            location[N_PACKET]=CARRIED;
            p_ret("You unscrew the cap and tip the packet upside down. The "
              "liquid inside pours out onto the ground and runs away in a "
              "long stream.\n");
          }
        if(n==N_WATER)
          {
            location[N_WATER]=NOWHERE;
            location[N_BUCKET]=CARRIED;
            p_ret("You tip over the bucket and pour the water onto the "
              "ground. It makes a large damp patch which slowly evaporates "
              "dry.\n");
          }
        p_ret("You can't empty that!\n");
      case V_THROW:
        if(location[n]!=CARRIED) p_ret("You're not holding it.\n");
        if(cp==L_STREAM_WEST||cp==L_STREAM_EAST)
          {
            location[n]=NOWHERE;
            p_ret("It falls into the stream with a splash and is lost.\n");
          }
        if(cp==L_CHASM_WEST||cp==L_CHASM_EAST)
          {
            location[n]=NOWHERE;
            p_ret("It plummets into the depths of the earth and is lost "
              "forever.\n");
          }
        if(cp==L_TREETOP)
          {
            location[n]=L_TREE;
            p_ret("You fling it from the branches of the tree. It lands on "
              "the ground below.\n");
          }
        location[n]=cp;
        p_ret("You hurl it a short distance. It lands on the ground. It "
          "doesn't seem damaged by the experience.\n");
      case V_UNLOAD:
        if(location[n]!=CARRIED) p_ret("You're not holding it.\n");
        if(n==N_GUN_UNLOADED) p_ret("It's not loaded.\n");
        if(n==N_GUN_LOADED)
          {
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=location[N_BULLET]=CARRIED;
            p_ret("You remove the bullet from the gun.\n");
          }
        p_ret("You can't unload that!\n");
      case V_FIRE:
FIRE:   if(location[n]!=CARRIED) p_ret("You're not holding it.\n");
        if(n==N_GUN_UNLOADED)
          p_ret("You pull the trigger of the gun. There is a loud click but "
            "nothing else happens.\n");
        if(n==N_GUN_LOADED)
          {
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=CARRIED;
            p_ret("You pull the trigger of the gun. There is a loud BANG. "
              "The bullet doesn't seem to have hit anything.\n");
          }
        p_ret("You can't fire that!\n");
      case V_SHOOT:
        if(n==N_BULLET||n==N_GUN_LOADED||n==N_GUN_UNLOADED) goto FIRE;
        if(location[N_GUN_UNLOADED]!=CARRIED&&location[N_GUN_LOADED]!=CARRIED)
          p_ret("You don't seem to be armed.\n");
        if(location[N_GUN_UNLOADED]==CARRIED)
          p_ret("The gun isn't loaded.\n");
        if(n==N_WEREWOLF)
          {
            werewolf_dead=2;
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=CARRIED;
            p_ret("You aim quickly and fire. There is a loud BANG and the "
              "werewolf stops dead in his tracks, a bloody hole in his chest "
              "and a look of surprise and indignation on his face. He slumps "
              "to the ground. You expect to see the body change back to "
              "human form as in the movies but what actually happens is that "
              "it melts away into thin air.\n");
          }
        if(n==N_GLASS)
          {
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=CARRIED;
            p_ret("You fire the gun at the pane of glass. The bullet smashes "
              "through it, making a hole about an inch wide. Now all you "
              "need to get past this obstacle is some way to make yourself "
              "an inch tall.\n");
          }
        if(n==N_OGRE)
          {
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=CARRIED;
            p_ret("You aim the gun at the ogre and fire, blowing a hole "
              "clean through his chest. This seems to make him a bit peeved, "
              "and the muzzle flash revealed your position. With astonishing "
              "endurance, he manages to rip you apart before collapsing and "
              "dying.\n");
          }
        if(n==N_DOOR)
          {
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=CARRIED;
            p_ret("You fire the gun at the door lock. Unfortunately, silver "
              "is not the best material for an armour-piercing bullet; the "
              "mechanism is dented but not ruined.\n");
          }
        if(n==N_VAMPIRE)
          {
            location[N_GUN_LOADED]=NOWHERE;
            location[N_GUN_UNLOADED]=CARRIED;
            print("You fire the gun at the vampire. He staggers, but does "
              "not fall. He treats you to a fangy grin and opens his "
              "cloak, showing you his bulletproof vest.\n");
          }
        p_ret("There's no point in shooting at that.\n");
      case V_KILL:
KILL:   if(n==N_TREE||n==N_FALLEN_TREE) goto CUT;
        if(n==N_PLANT||n==N_WEEDS) goto SPRAY;
        if(n==N_OGRE)
          {
            print("You launch a heroic attack on the monstrous ogre. "
              "Unfortunately, your attack doesn't succeed.\n");
            ogre_death();
            return;
          }
        p_ret("What's the point in attacking that?\n");
      case V_BREAK:
        if(n==N_GLASS)
          {
            if(glass_cut) p_ret("That's already been done.\n");
            p_ret("You attempt to smash the glass but it seems to be a "
              "particularly tough variety. Your efforts are in vain.\n");
          }
        if(n==N_MIRROR)
          {
            location[N_MIRROR]=NOWHERE;
            p_ret("You smash the mirror into a thousand minute shards of "
              "glass which scatter on the ground and are lost.\n");
          }
        p_ret("Senseless vandalism will get you nowhere.\n");
      case V_KICK:
        if(n==N_OGRE) goto KILL;
        p_ret("Taking out your frustration on the objects around you is a "
          "good way to lower your blood pressure, but it doesn't seem to be "
          "getting you anywhere.\n");
      case V_FUCK:
        p_ret("It says something about human nature, doesn't it, that "
          "practically everyone who writes an adventure game puts in a "
          "response to swear words.\n");
      case V_VERBOSE:
        mode=M_VERBOSE;
        p_ret("Verbose descriptions now active.\n");
      case V_BRIEF:
        mode=M_BRIEF;
        p_ret("Brief descriptions now active.\n");
      case V_NORMAL:
        mode=M_NORMAL;
        p_ret("Normal descriptions now active.\n");
      case V_SWIM:
        if(cp!=L_STREAM_WEST && cp!=L_STREAM_EAST)
          p_ret("There doesn't seem to be a single halfway decent swimming "
            "pool in this place.\n");
        game_over=TRUE;
        p_ret("You jump into the stream and start to swim across. This would "
          "probably have been a more sensible thing to do were it not for "
          "the inconvenient fact that the stream is full of piranha fish. "
          "You can probably imagine the rest (teeth, screams, blood, "
          "stripped bones etc). Anyway, suffice it to say that you end up "
          "fairly dead.\n");
    }
}

ubyte oldn=0;           // Stores previous turn's noun for IT facility
ubyte noun_it=0;        // Flags whether used IT this turn

void _main(void)
{
  randomize();          // Initialize for all games
  mode=M_NORMAL;
  FontSetSys(F_4x6);
  clrscr();
  do
    {
      init();           // Initialize for this game
      desc_loc();       // and describe the first location to the player
      do
	{
          status();
          if(game_over) break;
          get_command();
          do_command();
	}
      while(!game_over);
      print("Do you want to play again? (Y/N) ");
    }
  while(yesno());       // More games until player replies no
}

void init(void)
{
  int i;
  cp=9;
  look=1; // Indicates that at start the player just wants to see the location
          // description; he hasn't moved into it from an adjoining location

  game_over=scr_ct=0;
  for(i=2;i<=12;i++) flag_base[i]=FALSE;
  for(i=1;i<=NUM_LOC;i++) visited[i]=FALSE;
  for(i=1;i<=NUM_OBJ;i++) moved[i]=!*init_obj[i];
    // If no initial description, mark it as moved from the start
  for(i=1;i<=NUM_OBJ;i++) location[i]=init_loc[i];
  for(i=1;i<20;i++) putchar('\n');
  print("Welcome to Cave Adventure.\nOriginal by Russell Wallace 1988.\n"
    "Ported to TI-89 & TI-92+ by Zeljko Juric.\n\nDo you want instructions "
    "(Y/N)? ");
  if(yesno())
    {
      putchar('\n');
      print("Cave Adventure is originally written in Basic on the Commodore "
        "64 back in the early eighties, expanded and rewritten in C on the "
        "Amiga and the C version is ported to MS-DOS. Due to high portability"
        "of C language, you are now able to play it on your TI calculator."
        "\n\nThe plot is simple: You start the game near an underground "
        "cave system which is rumoured to contain fabulous treasures. It is "
        "also rumoured to contain a selection of refugees from low-grade "
        "horror movies who took up residence after the original owners moved "
        "out, along with similar beings from the pages of Tolkien and "
        "suchlike. Your mission is to become as rich as possible by "
        "collecting objects of the first category while avoiding death at "
        "the hands of those of the second category. The scoring system is "
        "based on results: rather than getting points for accomplishing "
        "tasks, you get points for collecting treasures and bringing them to "
        "the designated collection point (which will be marked in the game). "
        "Input is verb-noun only (when the original version was written, that"
        " was the state of the art of parser technology, and there wasn't "
        "much point in writing a more advanced parser for this version, "
        "since the puzzles are mostly the same as in the original and don't "
        "require more than verb-noun). The advanced features available are "
        "GET ALL, DROP ALL and IT.\n\nHave fun!\n\nPress any key to start.");
      getch();
      scr_ct=0;
      putchar('\n');
    }
  putchar('\n');
}

void get_command(void)
{
  int i,j,length;
  char str[100];
  char *p,*q;     // Temporary storage for pointer to typed-in verb and noun
LOOP:
  print("What now? ");
  getline();
  length=strlen(buffer);
  p=buffer;
  while(*p==' ') p++;
  if(!*p) goto LOOP;                     // Ignore null command
  putchar('\n');
  v=n=0;
  for(i=0;p[i]&&p[i]!=' ';i++);          // Put stop at end of verb for strcmp
  p[i]=0;
  for(v=1;v<=NUM_VERB&&strcmp(verb[v],p);v++);
  if(v>NUM_VERB)
    {
      for(j=0;j<NUM_VERB_SYN;j++)
        if(!strcmp(verb_syn[j],p))
          {
            v=verb_syn_num[j];
            goto FOUND_VERB_SYN;
          }
      sprintf(str,"I don't understand the word \"%s\".\n",p); print(str);
      goto LOOP;
    }
FOUND_VERB_SYN:
  if(i!=length)      // If there was something after the verb (e.g. a noun)
    {
      for(q=p+i+1;*q==' ';q++);  // Ignore spaces before noun
      if(!*q) goto NO_NOUN;      // Now have we ignored everything, or was 
      i=0;                       // there some text as well as the spaces?
      do                         // OK, now wipe out any spaces after the noun
	{
          if(q[i]==' ') q[i]=0;
          i++;
        } while(q[i]);
      noun_it=FALSE;
      if(!strcmp(q,"it"))
	{
          if(!oldn)
	    {
              print("I don't understand what you're referring to.\n");
              goto LOOP;
	    }
          noun_it=TRUE;
          n=oldn;
          goto GOT_NOUN;
	}
      for(n=1;n<=NUM_NOUN&&strcmp(noun[n],q);n++);
      if(n>NUM_NOUN)
	{
          for(j=0;j<NUM_NOUN_SYN;j++)
            if(!strcmp(noun_syn[j],q))
              {
                n=noun_syn_num[j];
                goto FOUND_NOUN_SYN;
              }
          sprintf(str,"I don't understand the word \"%s\".\n",q); print(str);
          goto LOOP;
	}
FOUND_NOUN_SYN:
      oldn=n;
GOT_NOUN:
      if(location[n]==NOWHERE)  // Find out whether referring to the lit or 
        {                       // unlit torch, full or empty bucket etc.
          switch(n)
	    {
              case N_TORCH_UNLIT:
                n=N_TORCH_LIT; break;
              case N_WEEDKILLER:
                n=N_PACKET; break;
              case N_BUCKET:
                n=N_WATER; break;
              case N_GUN_UNLOADED:
                n=N_GUN_LOADED; break;
              case N_SAFE_LOCKED:
                n=N_SAFE_OPEN; break;
              case N_TREE:
                n=N_FALLEN_TREE; break;
              case N_OIL:
                n=N_CAN;
            } // End of switch
        } // End of condition if location[n]==NOWHERE
      if(n==N_ALL&&v!=V_GET&&v!=V_DROP)
	{
          print("You can't use the word ALL with the verb \"");
          print(p);
          print("\".\n");
          goto LOOP;
	}
      if((n==N_GLASS&&cp!=L_GLASS)||(n==N_WEEDS&&cp!=L_WEEDS)||
        (n==N_WIRE&&cp!=L_WIRE)||(n==N_GATE && cp!=L_GATE)||
        (n==N_WEREWOLF&&(cp!=L_WEREWOLF||werewolf_dead==2))||
        (n==N_SIGN&&cp!=L_SIGN)||(n==N_PLANT&&cp!=L_PLANT&&cp!=L_TOP_PLANT)||
        (n==N_OGRE&&cp!=L_OGRE)||(n==N_DOOR&&cp!=L_DOOR)||
        (n==N_CHASM&&cp!=L_CHASM_WEST&&cp!=L_CHASM_EAST)||
        (n==N_STREAM&&cp!=L_STREAM_WEST&&cp!=L_STREAM_EAST)||
        (n==N_VAMPIRE&&cp!=L_VAMPIRE)||(n==N_BASILISK&&cp!=L_BASILISK)||
        (n<=NUM_OBJ&&location[n]!=cp&&location[n]!=CARRIED&&
        location[n]!=WORN))
          {
            print("I don't see any ");
            if(!noun_it)
              {
                print(q); putchar(' ');
              }
            print("here.\n");
	    goto LOOP;
          }
    } // End of condition if noun exists
NO_NOUN:
  if(!n&&need_obj[v])                                 
    {                                                 
      print("I don't understand what you want to ");  // Protest if the verb 
      print(p);                                       // has to have a noun  
      print(".\n");                                   // and none has been 
      goto LOOP;                                      // given...
    }
}

// If you have any comment, suggestions, questions or bug reports, mail me at:
// zjuric@utic.net.ba

// Zeljko Juric
// Sarajevo
// Bosnia & Herzegovina
